#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu May 13 20:13:56 2021

@author: Kevin Hank

How to use:
python pismInputFilesDist.py np res fileIn
np = line of parameter vector in parameter file (paramFile below, e.g., 1)
res = grid resolution in m (e.g., 25000)
fileIn = name of the netcdf file the fields get stored in (path needs to be provided within this script)
"""

import numpy as np    
import matplotlib.pyplot as plt
import netCDF4 as nc
import os
import sys
import matplotlib.colors as colors
from itertools import repeat

plt.rcParams.update({'font.size': 28})
plt.close()

# plot the input fields?
fieldPlot = False

# define setup
paramVec = int(sys.argv[1])-1 # -1 because of python notation (start at 0), in this way run number matches line number in param file   
res = int(int(sys.argv[2]))   
scal = int(50000/res)

nj = 60*scal        # number of rows (fortran)
ni = 60*scal
nt = 101

xCenter = (nj)*res/2
yCenter = np.copy(xCenter)
R = np.copy(xCenter)

# parameter file
paramFile = '/home/khank/PISMv2.0.2/LISsq/Params/params.Range.pismCircSQsoft0p5to1p5'
soft, hard, Bmax, Sb, Tmin, St = np.loadtxt(paramFile, skiprows=paramVec, max_rows=1)

path = '/home/khank/PISMv2.0.2/LISsq/Input/'
fn = path+str(sys.argv[3])

try:
    os.remove(fn)
except:
    pass

ds = nc.Dataset(fn, 'w', format='NETCDF4')

time = ds.createDimension('time', None)
y = ds.createDimension('y', nj)
x = ds.createDimension('x', ni)

times = ds.createVariable('time', 'f8', ('time',))
ys = ds.createVariable('y', 'f8', ('y',))
xs = ds.createVariable('x', 'f8', ('x',))

times[:] = 0
xAr = np.arange(res/2, res*nj+res/2, res)
yAr = np.arange(res/2, res*ni+res/2, res)
xs[:] = xAr
ys[:] = yAr

# metadata
times.standard_name = 'time'
times.long_name = "time"
times.units = "seconds since 1-1-1"
times.calendar = "365_day"
times.axis = "T"

xs.standard_name = "projection_x_coordinate" 
xs.long_name = "X-coordinate in Cartesian system" 
xs.units = "km" 
xs.axis = "X" 

ys.standard_name = "projection_y_coordinate" 
ys.long_name = "Y-coordinate in Cartesian system" 
ys.units = "km" 
ys.axis = "Y" 


# create input fields and assign metadata
topg = ds.createVariable('topg', 'f8', ('time', 'y', 'x',))
topg.standard_name = "bedrock_altitude" 
topg.long_name = "bedrock surface elevation" 
topg.units = "m" 
topg.coordinates = "lat lon" 
topg.pism_intent = "model_state" 


climatic_mass_balance = ds.createVariable('climatic_mass_balance', 'f8', ('time', 'y', 'x',))
climatic_mass_balance.standard_name = "land_ice_surface_specific_mass_balance_flux" 
climatic_mass_balance.long_name = "surface mass balance (accumulation/ablation) rate" 
climatic_mass_balance.units = "kg m-2 year-1" 
climatic_mass_balance.coordinates = "lat lon" 
climatic_mass_balance.pism_intent = "climate_forcing"

ice_surface_temp = ds.createVariable('ice_surface_temp', 'f8', ('time', 'y', 'x',))
ice_surface_temp.long_name = "temperature of the ice at the ice surface but below firn processes" 
ice_surface_temp.units = "Kelvin" 
ice_surface_temp.coordinates = "lat lon" 
ice_surface_temp.pism_intent = "climate_forcing" 

tillphi = ds.createVariable('tillphi', 'f8', ('time', 'y', 'x',))


# calculate distance from center
dist = np.zeros((nj,ni))
for i in range(nj):
    for j in range(ni):
        tmp = np.sqrt((xAr[i]-xCenter)**2 + (yAr[j]-yCenter)**2)
        if tmp<R:
            dist[i,j] = tmp
        else:
            dist[i,j] = R

# rescale dist output (eq for Tsurf and agdm are for km not m while PISM takes m as input)
dist = dist/1000
            
Bsurf = Bmax - Sb * dist**5
agdmS = np.zeros((1,nj,ni))
agdmS[0,:,:] = Bsurf

Tsurf = Tmin + St*dist**3
TsurfS = np.zeros((1,nj,ni))
TsurfS[0,:,:] = Tsurf

# topography -----------------------------------------
minDepth = 200
maxDepth = 0

topo = np.zeros((ni,nj))+3
topo[10*scal:50*scal,10*scal:50*scal] = 1
topo[22*scal:38*scal,18*scal:34*scal] = 2 # HS and HB
topo[28*scal:32*scal,34*scal:] = 2

topo = np.where(topo==1,minDepth,topo)
topo = np.where(topo==2,maxDepth,topo)
topo = np.where(topo==3,minDepth,topo)

uh = np.linspace(maxDepth,minDepth,6)
lh = np.linspace(minDepth,maxDepth,6)

if scal>1:
    uh = [iScal for item in uh for iScal in repeat(item, scal)]
    lh = [iScal for item in lh for iScal in repeat(item, scal)]

# HS and HB
for i in range(18*scal,60*scal):
    if i<34*scal:
        topo[17*scal:23*scal,i] = lh
        topo[37*scal:43*scal,i] = uh
    elif i>=34*scal:
        topo[23*scal:29*scal,i] = lh
        topo[31*scal:37*scal,i] = uh
for j in range(16*scal,44*scal):
    ls = np.linspace(minDepth,topo[j,19*scal],6)
    rs = np.linspace(topo[j,33*scal-1],topo[j,39*scal],6)
    if scal==1:
        topo[j,13*scal:19*scal] = ls
        if j<28*scal or j>=32*scal:
            topo[j,33*scal:39*scal] = rs
    else:
        topo[j,13*scal:19*scal] = [iScal for item in ls for iScal in repeat(item, scal)]
        if j<28*scal or j>=32*scal:
            topo[j,33*scal:39*scal] = [iScal for item in rs for iScal in repeat(item, scal)]
            
topoS = np.zeros((1,nj,ni)) # flat topography
#topoS[0,:,:] = topo         # uncomment for above topography
            
# tillphi -----------------------------------------
tillphi = np.zeros((ni,nj))
tillphi[10*scal:50*scal,10*scal:50*scal] = 1
tillphi[22*scal:38*scal,18*scal:34*scal] = 2
tillphi[28*scal:32*scal,34*scal:] = 2 # extend soft sediment area to eastern end of model domain

tillphi = np.where(tillphi==1,hard,tillphi)
tillphi = np.where(tillphi==2,soft,tillphi)
tillphi = np.where(tillphi==0,hard,tillphi)

uh = np.linspace(soft,hard,6)
lh = np.linspace(hard,soft,6)
if scal>1:
    uh = [iScal for item in uh for iScal in repeat(item, scal)]
    lh = [iScal for item in lh for iScal in repeat(item, scal)]

for i in range(18*scal,60*scal): # extend soft sediment area to eastern end of model domain
    if i<34*scal:
        tillphi[17*scal:23*scal,i] = lh
        tillphi[37*scal:43*scal,i] = uh
    elif i>=34*scal:
        tillphi[23*scal:29*scal,i] = lh
        tillphi[31*scal:37*scal,i] = uh
for j in range(16*scal,44*scal):
    ls = np.linspace(hard,tillphi[j,19*scal],6)
    rs = np.linspace(tillphi[j,33*scal-1],tillphi[j,39*scal],6)
    if scal==1:
        tillphi[j,13*scal:19*scal] = ls
        if j<28*scal or j>=32*scal:
            tillphi[j,33*scal:39*scal] = rs
    else:
        tillphi[j,13*scal:19*scal] = [iScal for item in ls for iScal in repeat(item, scal)]
        if j<28*scal or j>=32*scal:
            tillphi[j,33*scal:39*scal] = [iScal for item in rs for iScal in repeat(item, scal)]

tillphiS = np.zeros((1,nj,ni))
tillphiS[0,:,:] = tillphi

# assign fields
ds.variables['topg'][:] = topoS
ds.variables['climatic_mass_balance'][:] = agdmS
ds.variables['ice_surface_temp'][:] = TsurfS
ds.variables['tillphi'][:] = tillphiS

ds.close()

# plots -----------------------------------------
if fieldPlot == True:
    xp = np.linspace(0,ni*res,ni+1)
    yp = np.linspace(0,nj*res,nj+1)

    boundsagdm = np.arange(-10000,0,1000)
    boundsagdm = np.append(boundsagdm,np.arange(0,260+10,10)) # arange does not include the endpoint
    normBoundsagdm = colors.BoundaryNorm(boundaries=boundsagdm,ncolors=256)

    my_dpi=96
    fig, ax = plt.subplots(1,1,figsize=(1080/my_dpi, 1080/my_dpi), dpi=my_dpi)
    
    agdmplot = plt.pcolor(xp,yp,agdmS[0,:,:],cmap='jet',norm=normBoundsagdm)
    cbar = plt.colorbar(agdmplot)
    plt.title('Surface mass balance')

    # Tsurfplot = plt.pcolor(xp,yp,TsurfS[0,:,:],cmap='jet')
    # cbar = plt.colorbar(Tsurfplot)
    # plt.title('Surface temperature')

    # tillphiplot = plt.pcolor(xp,yp,tillphi,cmap='jet',edgecolors='w')
    # cbar = plt.colorbar(tillphiplot)
    # plt.title('Till friction angle')

    # tillphi = np.where(tillphi==soft,0,1)

    # newcmp = ListedColormap(['tab:orange','darkgray'])
    # psm=ax.pcolor(xp,yp,tillphi,edgecolors='w',cmap=newcmp)
    # cbar = fig.colorbar(psm,ax=ax,ticks=range(2))
    # cbar.ax.set_yticklabels(['soft bed','hard bed'])
    # plt.xlim(-res/2,ni*res+res/2)
    # plt.ylim(-res/2,nj*res+res/2)
    # plt.axis('equal')
    # plt.xlabel('m')
    # plt.ylabel('m', labelpad=(-0))
    # plt.title('Idealized North American Geometry', y=1.05)

    def sedimentLines():
        col = 'm'
        [x1,x2,x3] = 900, 1700, 2500                
        [y1,y2,y3,y4] = 1100, 1400, 1600, 1900 #750, 1050, 1250, 1550  
        plt.hlines(y1,x1,x2,colors=col)
        plt.hlines(y4,x1,x2,colors=col)
        plt.vlines(x1,y1,y4,colors=col)
        plt.vlines(x2,y1,y2,colors=col)
        plt.vlines(x2,y3,y4,colors=col)
        plt.hlines(y2,x2,x3,colors=col)
        plt.hlines(y3,x2,x3,colors=col)
    sedimentLines()

    plt.axis('equal')
    plt.xlabel('km')
    plt.ylabel('km')
    plt.grid()

    plt.show()
